function [sc,fem_dat_sh,fem_dat_be,bc] = double_bottom(fileID,xof,g_lbwf,...
    g_Htt,db_ns,db_W,g_wss,sc,wmh,section,db_plt_os,db_plt_tt,...
    db_plt_wf,db_sti_wf,db_plt_str,db_sti_str,db_sti_os,db_sti_tt,...
    fem_dat_sh,fem_dat_be,db_plt_wf_s,bc)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function for generation of PATRAN session file commands which will      %
% generate a double bottom section when run.                              %
%                                                                         %
% Written by Halvor L. Aga as part of the work concerning                 %
% "Assessement if structural requirements related to LNG fuel tanks."     %
% Deadline 10. June 2013                                                  %
% Has been made by modification of recorded .ses commands in PATRAN.      %
%                                                                         %
% Input:                                                                  %
% fileID - file identification number of the file for writing             %
% xoff - Offset in x direction for the section					          %
% g_lbwf - Length between the web frames					              %
% g_Htt  - Height of the tank top						                  %
% db_ns - number of stiffeners in each distance between the longitudinal  %
%         girders                                                         %
% db_W - Width between each of the longitudinal girders				      %
% g_wss - Width of the side section						                  %
% sc - Counter for number of created surfaces					          %
% wmh - Width of a manhole							                      %
% section - Information of placement of the section			              %
% db_plt_os - property numbers for plating in outer skin 			      %
% db_plt_tt - property numbers for plating in tank top	 			      %
% db_plt_wf - property numbers for plating in web frame 			      %
% db_plt_str- property numbers for plating in stringers 			      %
% db_sti_str- property numbers for stiffeners in stringers 			      %
% db_sti_os - property numbers for stiffeners in outer skin 			  %
% db_sti_tt - property numbers for stiffeners in tank top	 	    	  %
% fem_dat_sh - matrix containing data regarding shell elements			  %
% fem_dat_be - matrix containing data regarding beam elements			  %
% db_plt_wf_s - property numbers for plating in web frame where holes or  %
%               manholes                                                  %			 					  %
% bc - matrix containing data regarding boundary conditions			      %
%										                                  %
% Output:									                              %
% sc - Counter for number of created surfaces					          %
% fem_dat_sh - matrix containing data regarding shell elements			  %
% fem_dat_be - matrix containing data regarding beam elements			  %
% bc - matrix containing data regarding boundary conditions			      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Determination of femc, finite elements counter
if (size(fem_dat_sh,2)==1); femc_sh = 0;
else femc_sh = size(fem_dat_sh,1);
end
% Determination of femc_be, finite elements counter for beam elements
if (size(fem_dat_be,2)==1); femc_be = 0;
else femc_be = size(fem_dat_be,1);
end
% Determination of bcc, counter for boundary conditions
if (size(bc,2)==1); bcc = 0;
else bcc = size(bc,1);
end




xofm = [xof, xof+(g_lbwf-800)/2, xof+(g_lbwf+800)/2];
lbwfm = [(g_lbwf-800)/2, 800, (g_lbwf-800)/2];

for b = 1:3
    i_xof = xofm(b);
    i_l = lbwfm(b);
    pc = 0; % Point counter
    cc = 0; % Curve counter
    sco = sc ; % Store number of surfaces generated
    %--------------------------------------------------------------------------
    % Point generation
    
    % Origo
    fprintf(fileID,'STRING asm_create_grid_xyz_created_ids[VIRTUAL]\n');
    pc = pc + 1;
    fprintf(fileID,'asm_const_grid_xyz( "%d", "[%d 0 0]", "Coord 0",  @\n',pc,i_xof);
    fprintf(fileID,'asm_create_grid_xyz_created_ids )\n');
    % Arc
    % Calculate position of points in arc
    if g_wss <= g_Htt;
        y = 0;
        z = g_wss-g_Htt;
    else
        y = g_Htt;
        z = -g_Htt;
    end
    pc = pc + 1;
    fprintf(fileID,'asm_const_grid_xyz( "%d", "[%d %d %d]",  @\n',pc,i_xof,y,z);
    fprintf(fileID,'"Coord 0", asm_create_grid_xyz_created_ids )\n');
    pc = pc + 1;
    fprintf(fileID,'asm_const_grid_xyz( "%d", "[%d %d %d]",  @\n',pc,i_xof,g_wss,-g_Htt);
    fprintf(fileID,'"Coord 0", asm_create_grid_xyz_created_ids )\n');
    % Double hull structure
    % Along stringers on tank top
    % Calculation of y values for first point
    y = g_wss;
    pc = pc+1;
    fprintf(fileID,'asm_const_grid_xyz( "%d", "[%d %d 0]",  @\n',pc,i_xof,y);
    fprintf(fileID,'"Coord 0", asm_create_grid_xyz_created_ids )\n');
    for i = 1:length(db_ns)
        for j = 1:db_ns(i)+1
            % Calculating point numbers and position
            y = y + db_W(i)/(db_ns(i)+1);
            pc = pc + 1;
            % Printing necessary commands for generation of point i
            fprintf(fileID,'asm_const_grid_xyz( "%d", "[%d %d 0]",  @\n',pc,i_xof,y);
            fprintf(fileID,'"Coord 0", asm_create_grid_xyz_created_ids )\n');
        end
    end
    
    % Point generation finished
    %----------------------------------------------------------------------------------
    % Curve generation
    
    % Radius
    R = min(g_Htt,g_wss);
    if g_Htt<=g_wss
        cc = cc + 1;
        fprintf(fileID,'STRING sgm_create_curve_2d_created_ids[VIRTUAL]\n');
        fprintf(fileID,'sgm_const_curve_2d_arc2point_v2( "%d", 2, %d, FALSE, FALSE, 1, "Coord 0.1",  @\n',cc,R);
        fprintf(fileID,'"", "point 1", "point 2", FALSE, sgm_create_curve_2d_created_ids )\n');
        cc = cc+1;
        fprintf(fileID,'STRING asm_line_2point_created_ids[VIRTUAL]\n');
        fprintf(fileID,'asm_const_line_2point( "%d", "point 2", "point 3", 0, "", 50., 1,  @\n',cc);
        fprintf(fileID,'asm_line_2point_created_ids )\n');
    else
        cc = cc + 1;
        fprintf(fileID,'STRING sgm_create_curve_2d_created_ids[VIRTUAL]\n');
        fprintf(fileID,'sgm_const_curve_2d_arc2point_v2( "1", 2, %d, FALSE, FALSE, 1, "Coord 0.1",  @\n',R);
        fprintf(fileID,'"", "point 2", "point 3", FALSE, sgm_create_curve_2d_created_ids )\n');
        cc = cc+1;
        fprintf(fileID,'STRING asm_line_2point_created_ids[VIRTUAL]\n');
        fprintf(fileID,'asm_const_line_2point( "%d", "point 1", "point 2", 0, "", 50., 1,  @\n',cc);
        fprintf(fileID,'asm_line_2point_created_ids )\n');
    end
    cc = cc+1;
    fprintf(fileID,'asm_const_line_2point( "%d", "point 1", "point 4", 0, "", 50., 1,  @\n',cc);
    fprintf(fileID,'asm_line_2point_created_ids )\n');
    cc = cc+1;
    fprintf(fileID,'asm_const_line_2point( "%d", "point 3", "point 4", 0, "", 50., 1,  @\n',cc);
    fprintf(fileID,'asm_line_2point_created_ids )\n');
    
    % Tank top and outer skin
    for i = 1:sum(db_ns)+length(db_ns)
        cc=cc+1;
        fprintf(fileID,'asm_const_line_2point( "%d", "point %d", "point %d", 0, "", 50., 1,  @\n',cc,i+3,i+4);
        fprintf(fileID,'asm_line_2point_created_ids )\n');
    end
    
    % Curve generation finished
    %----------------------------------------------------------------------------------
    % Surface generation
    
    % Web frame
    fprintf(fileID,'STRING sgm_sweep_surface_e_created_ids[VIRTUAL]\n');
    % Decide which panel is the first with manhole
    firstbrake = db_ns(1)+1+floor((db_ns(2)+1)/2)-ceil(wmh/2/(db_W(2)/(db_ns(2)+1)))+1+sco;
    % Decide which panel is the last with manhole
    lastbrake  = db_ns(1)+1+floor((db_ns(2)+2)/2)+ceil(wmh/2/(db_W(2)/(db_ns(2)+1)))+sco;
    % Between first and last longitudinal girder
    count = 4;
    for i = 1:length(db_ns)
        for j = 1:db_ns(i)+1
            sc = sc+1;
            count = count + 1;
            fprintf(fileID,'sgm_const_surface_extrude( "%d", "<0 0 %d>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,-g_Htt);
            fprintf(fileID,' "Curve %d", sgm_sweep_surface_e_created_ids )\n',count);
            % Breaking for easier meshing
            coord = [i_xof, g_wss+sum(db_W(1:i-1))+(j-1)*db_W(i)/(db_ns(i)+1), -(g_Htt/2-325);
                i_xof, g_wss+sum(db_W(1:i-1))+(j-1)*db_W(i)/(db_ns(i)+1), -(g_Htt/2+325) ;
                i_xof, g_wss+sum(db_W(1:i-1))+j*db_W(i)/(db_ns(i)+1), -(g_Htt/2-325) ;
                i_xof, g_wss+sum(db_W(1:i-1))+j*db_W(i)/(db_ns(i)+1), -(g_Htt/2+325) ];
            sc = prep_string(fileID,coord,sc,1,sc);
            
            % Store finite element data if not the first section
            % Vary properties
            for k = 1:3
                if section ~=1 && b == 1 && sc<=3*lastbrake+2-2*sco && sc>=3*firstbrake-2*sco
                    femc_sh = femc_sh + 1;
                    fem_dat_sh(femc_sh,1) = sc-k+1; % Surface number
                    fem_dat_sh(femc_sh,2) = db_plt_wf_s(1); % Property number
                    fem_dat_sh(femc_sh,3) = 1; % Mesh fineness
                    if j~=1
                        femc_be = femc_be +1;
                        fem_dat_be(femc_be,1) = sc-k+1; % Surface number
                        fem_dat_be(femc_be,2) = 1;  % Curve side number
                        fem_dat_be(femc_be,3) = db_sti_wf(i);  % Property number
                        fem_dat_be(femc_be,4) = 1; % Mesh fineness
                    end
                elseif  section ~=1 && b == 1 && (sc == sco+sum(db_ns)+length(db_ns)-ceil(db_ns(length(db_ns))/2)...
                        ||sc == sco+sum(db_ns)+length(db_ns)-ceil(db_ns(length(db_ns))/2)+1);
                    femc_sh = femc_sh + 1;
                    fem_dat_sh(femc_sh,1) = sc-k+1;
                    fem_dat_sh(femc_sh,2) = db_plt_wf_s(2);
                    fem_dat_sh(femc_sh,3) = 1;
                    if j~=1
                        femc_be = femc_be +1;
                        fem_dat_be(femc_be,1) = sc-k+1;
                        fem_dat_be(femc_be,2) = 1;
                        fem_dat_be(femc_be,3) = db_sti_wf(i);
                        fem_dat_be(femc_be,4) = 1;
                    end
                elseif section ~=1 && b == 1
                    femc_sh = femc_sh + 1;
                    fem_dat_sh(femc_sh,1) = sc-k+1;
                    fem_dat_sh(femc_sh,2) = db_plt_wf(i+1);
                    fem_dat_sh(femc_sh,3) = 1;
                    if j~=1
                        femc_be = femc_be +1;
                        fem_dat_be(femc_be,1) = sc-k+1;
                        fem_dat_be(femc_be,2) = 1;
                        fem_dat_be(femc_be,3) = db_sti_wf(i);
                        fem_dat_be(femc_be,4) = 1;
                    end
                end
            end
        end
    end
    
    % Webframe inside radius
    sc = sc + 1;
    fprintf(fileID,'STRING sgm_surface_4edge_created_ids[VIRTUAL]\n');
    fprintf(fileID,'sgm_const_surface_4edge( "%d", "Curve 1", "Curve 2", "Curve 3", @\n',sc);
    fprintf(fileID,'"Curve 4", sgm_surface_4edge_created_ids )\n');
    % Store finite element data if not the first section
    if section ~=1 && b == 1
        femc_sh = femc_sh + 1;
        fem_dat_sh(femc_sh,:) = [sc db_plt_wf(1) 1];
    end
    
    % Extrude section to surfaces from curves
    fprintf(fileID,'STRING sgm_sweep_surface_e_created_ids[VIRTUAL]\n');
    
    % Radius
    for i = 1:2
        sc = sc+1;
        fprintf(fileID,'sgm_const_surface_extrude( "%d", "<%d 0 0>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,i_l);
        fprintf(fileID,' "Curve %d", sgm_sweep_surface_e_created_ids )\n',i);
        % Store finite element data
        femc_sh = femc_sh + 1;
        fem_dat_sh(femc_sh,1) = sc;
        fem_dat_sh(femc_sh,2) =    db_plt_os(1) ;
         if  section == 2
            fem_dat_sh(femc_sh,3) = 2;
         else
             fem_dat_sh(femc_sh,3) = 1;
         end
        % Store boundary conditions if first or last section
        if section == 1 && b == 1
            bcc = bcc + 1;
            bc(bcc,1) = sc; % Surface number
            bc(bcc,2) = 4;  % Side number
            bc(bcc,3) = 1;  % Boundary condition type
        elseif section == 3 && b == 3
            bcc = bcc + 1;
            bc(bcc,1) = sc;
            bc(bcc,2) = 2;
            bc(bcc,3) = 1;
        end
    end
    
    % Tanktop
    count = 4;
    for i = 1:length(db_ns)
        for j = 1:db_ns(i)+1
            count = count + 1;
            sc = sc+1;
            fprintf(fileID,'sgm_const_surface_extrude( "%d", "<%d 0 0>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,i_l);
            fprintf(fileID,' "Curve %d", sgm_sweep_surface_e_created_ids )\n',count);
            femc_sh = femc_sh + 1;
            fem_dat_sh(femc_sh,:) = [sc db_plt_tt(i) 1];
            if j ~= 1
                femc_be = femc_be +1;
                fem_dat_be(femc_be,:) = [sc 1 db_sti_tt(i) 1];
            end
            if section == 1 && b == 1
                bcc = bcc + 1;
                bc(bcc,:) = [sc 4 1];
            elseif section == 3 && b == 3
                bcc = bcc + 1;
                bc(bcc,:) = [sc 2 1];
            end
        end
    end
    
    % Outer skin
    count = sco-2;
    for i = 1:length(db_ns)
        for j = 1:db_ns(i)+1
            count = count + 3;
            sc = sc + 1;
            fprintf(fileID,'sgm_const_surface_extrude( "%d", "<%d 0 0>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,i_l);
            fprintf(fileID,' "Surface %d.2", sgm_sweep_surface_e_created_ids )\n',count);
            femc_sh = femc_sh + 1;
            fem_dat_sh(femc_sh,:) = [sc db_plt_os(i+1) 1];
            if j ~= 1
                femc_be = femc_be +1;
                fem_dat_be(femc_be,:) = [sc,1,db_sti_os(1) 1];
            end
            if section == 1 && b == 1
                bcc = bcc + 1;
                bc(bcc,:) = [sc 4 1];
            elseif section == 3 && b == 3
                bcc = bcc + 1;
                bc(bcc,:) = [sc 2 1];
            end
        end
    end
    
    % Longitudinal girders
    count = sco + 1;
    for i = 1:length(db_ns)+1
        for k = 1:3
            sc = sc + 1;
            if i ~= 1 && k==1; count = count + 3*db_ns(i-1)+3;  end  
            if i ~= length(db_ns)+1
                fprintf(fileID,'sgm_const_surface_extrude( "%d", "<%d 0 0>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,i_l);
                fprintf(fileID,' "Surface %d.1", sgm_sweep_surface_e_created_ids )\n',count+k-1);   
                if k==3
                    femc_be = femc_be + 1;
                    fem_dat_be(femc_be,:) = [sc-1 3 db_sti_str(i) 1];
                    femc_be = femc_be + 1;
                    fem_dat_be(femc_be,:) = [sc-2 1 db_sti_str(i) 1];
                    femc_sh = femc_sh + 1;
                    fem_dat_sh(femc_sh,:) = [sc db_plt_str(2,i) 1];
                else
                    femc_sh = femc_sh + 1;
                    fem_dat_sh(femc_sh,:) = [sc db_plt_str(1,i) 1];                
                end
                if section == 1 && b == 1
                    bcc = bcc + 1;
                    bc(bcc,:) = [sc 4 1];
                elseif section == 3 && b == 3
                    bcc = bcc + 1;
                    bc(bcc,:) = [sc 2 1];
                end
            else             
                fprintf(fileID,'sgm_const_surface_extrude( "%d", "<%d 0 0>", 1., 0., "[0 0 0]", "Coord 0", @\n',sc,i_l);
                fprintf(fileID,' "Surface %d.3", sgm_sweep_surface_e_created_ids )\n',count-1+1-k);
                femc_sh = femc_sh + 1;
                fem_dat_sh(femc_sh,:) = [sc db_plt_str(1,i) 1];
                if section == 1 && b == 1
                    bcc = bcc + 1;
                    bc(bcc,:) = [sc 4 1];
                elseif section == 3 && b == 3
                    bcc = bcc + 1;
                    bc(bcc,:) = [sc 2 1];
                end               
            end
        end
    end
    
    % Surface generation finished
    %----------------------------------------------------------------------------------
    % Cleaning
    
    % Delete all points and curves used
    fprintf(fileID,'STRING asm_delete_point_deleted_ids[VIRTUAL]\n');
    fprintf(fileID,'asm_delete_point( "Point 1:2050", asm_delete_any_deleted_ids )\n');
    fprintf(fileID,'STRING asm_delete_curve_deleted_ids[VIRTUAL]\n');
    fprintf(fileID,'asm_delete_curve( "Curve 1:2050", asm_delete_curve_deleted_ids )\n');
    
end
end

