function [offsetm,bpname,finemesh] = mesh(fileID,fem_dat_sh,fem_dat_be,...
    bc,filesID)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function for generation of PATRAN session file commands which will      %
% generate neccessary commands for creating materials, sections and       %
% meshing in a ses script.                                                %
%                                                                         %
% Written by Halvor L. Aga as part of the work concerning                 %
% "Assessement if structural requirements related to LNG fuel tanks."     %
% Deadline 10. June 2013                                                  %
% Has been made by modification of recorded .ses commands in PATRAN.      %
%                                                                         %
% Build-up:                                                               %
%    1. Create sections                                                   %
%    2. Generate mesh for stiffeners                                      %
%    3. Sew surfaces                                                      %
%    4. Generate mesh for surfaces                                        %
%    5. Equivalence nodes                                                 %
%                                                                         %
% Input:                                                                  %
% fileID - file identification number of the file for writing             %
% fem_dat_sh - matrix containing data regarding shell elements			  %
% fem_dat_be - matrix containing data regarding beam elements			  %
% bc - matrix containing data regarding boundary conditions			      %
% filesID - Vector containing fileID of the open files that is used       %
%           througout the program                                         %
%										                                  %
% Output:                                                                 %
% offsetm - matrix containing data for beams about offset                 %
% bpname  - vector containing names of the beams in offsetm               %
% finemesh - Mesh size for fine meshing	                                  %
%										                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Nonlinear steel
fprintf(fileID,'material.create( "Analysis code ID", 10001,@\n');
fprintf(fileID,'"Analysis type ID", 1, "nlSteel",@\n');
fprintf(fileID,...
    '0, "Date: 06-Mar-13           Time: 20:42:28", "Isotropic", 1,@\n');
fprintf(fileID,'"Directionality", 1, "Linearity", 3, "Homogeneous",@\n');
fprintf(fileID,'0, "Elastoplastic", 3, "Model Options & IDs",@\n');
fprintf(fileID,'["Piecewise Linear(MAT24)", "Bilinear",@\n');
fprintf(fileID,'"N/A.", "Cowper Symonds", ""], [11026, 11028,@\n');
fprintf(fileID,'11022, 11031, 0], "Active Flag", 1, "Create",@\n');
fprintf(fileID,'10, "External Flag", FALSE, "Property IDs",@\n');
fprintf(fileID,'["Density", "Elastic Modulus", "Poisson Ratio",@\n');
fprintf(fileID,'"Yield Stress"], [16, 2, 5, 1011, 0],@\n');
fprintf(fileID,...
    '"Property Values", ["7.85e-9", "2.1e5", "0.3", "345", ""] )\n');

% Create sections
sectionID = filesID(3);
for i = 1:4 % Omitting lines
    fgets(sectionID);
end

% Retrieve number of shell sections
n=strread(fgets(sectionID));
fgets(sectionID); % omit line
% Shell sections
for i = 1:n
    %[section Name Materaltype Thickness]
    [a b c d] = strread(fgets(sectionID),'%s %s %s %f');
    fprintf(fileID,...
        'elementprops_create( "shell_%s_%s",@\n',char(a),char(b));
    fprintf(fileID,'51, 25, 35, 11004, 1, 20,@\n');
    fprintf(fileID,...
        '[13, 20, 36, 1004, 11044, 11136, 11027, 1011, 11182],@\n');
    fprintf(fileID,'[5, 2, 1, 1, 4, 3, 4, 1, 1],@\n');
    fprintf(fileID,...
        '["m:%s", "", "%f", "0.833", "", "5", "", "", ""],@\n',char(c),d);
    fprintf(fileID,'"" )\n');
    % Store property name
    v1 = {'shell_'};
    v2 = char(a);
    v3 = {'_'};
    v4 = char(b);
    spname(i) = strcat(v1,v2,v3,v4);
end

for i = 1:2 % Omit lines
    fgets(sectionID);
end

% Retrieve number of beam sections
m=strread(fgets(sectionID));
fgets(sectionID); % omitting line
offsetm = zeros(m,3);
% Beam sections
for i = 1:m
    % [section Name Materaltype Thickness Height Directionvector(xyz)]
    [a b c d e f g h m n o] = ...
        strread(fgets(sectionID),'%s %s %s %f %f %f %f %f %f %f %f');
    fprintf(fileID,'beam_section_create( @\n');
    fprintf(fileID,...
        '"FB_%3.1fx%3.1f_%s_%s", "BAR", ["%3.1f", "%3.1f"] )\n'...
        ,e,d,char(a),char(b),d,e);
    fprintf(fileID,...
        'elementprops_create( "beam_%s_%s",@\n',char(a),char(b));
    fprintf(fileID,'11, 38, 50, 11003, 1, 20, [39, 13, 6, 1004,@\n');
    fprintf(fileID,'11022, 11139, 11140, 1011, 11182], [11, 5, 2, 1, @\n');
    fprintf(fileID,...
        '4, 4, 4, 1, 1], ["FB_%3.1fx%3.1f_%s_%s", "m:%s",@\n'...
        ,e,d,char(a),char(b),char(c));
    fprintf(fileID,...
        '"<%f %f %f>", "0.833", "", "Center", "t=+1", "", ""], "" )\n',f,g,h);
    % Storing property name
    v1 = {'beam_'};
    v2 = char(a);
    v3 = {'_'};
    v4 = char(b);
    bpname(i,:) = strcat(v1,v2,v3,v4);
    % Store offset data
    offsetm(i,:) = [m n o];
end

%Generate mesh for stiffeners
%Collapse fem_dat_be
fem_dat_be_coll = collapse(fem_dat_be);
fprintf(fileID,'INTEGER fem_create_mesh_curve_num_nodes\n');
fprintf(fileID,'INTEGER fem_create_mesh_curve_num_elems\n');
fprintf(fileID,'STRING fem_create_mesh_c_nodes_created[VIRTUAL]\n');
fprintf(fileID,'STRING fem_create_mesh_c_elems_created[VIRTUAL]\n');
fgets(sectionID); % Omit line
% Read data for mesh fineness
[coursemesh finemesh]=strread(fgets(sectionID));
% Write commands for mesh on surface edges
for i = 1:size(fem_dat_be_coll,1)
    if fem_dat_be_coll(i,5) == 1;
        esize = coursemesh;
    elseif fem_dat_be_coll(i,5) == 2;
        esize = finemesh;
    end
    fprintf(fileID,...
        'fem_create_mesh_curv_1( "Surface %d:%d.%d", 16384, %d.,@\n'...
        ,fem_dat_be_coll(i,1),fem_dat_be_coll(i,2),...
        fem_dat_be_coll(i,3),esize);
    fprintf(fileID,'"Bar2", "#", "#", "Coord 0", "Coord 0",@\n');
    fprintf(fileID,'fem_create_mesh_curve_num_nodes,@\n');
    fprintf(fileID,'fem_create_mesh_curve_num_elems,@\n');
    fprintf(fileID,'fem_create_mesh_c_nodes_created,@\n');
    fprintf(fileID,'fem_create_mesh_c_elems_created )\n');
    fprintf(fileID,'fem_associate_elems_to_ep( "%s",@\n',...
        char(bpname(fem_dat_be_coll(i,4))));
    fprintf(fileID,'fem_create_mesh_c_elems_created,@\n');
    fprintf(fileID,'fem_create_mesh_curve_num_elems )\n');
end

% Collapse boundary condition matrix
bc = collapse(bc);
% Create boundary condition sets
for i = 1:size(bc,1)
    fprintf(fileID,'loadsbcs_create2( "%d_bc%d", "Displacement",@\n',...
        bc(i,4),i);
    fprintf(fileID,'"Nodal", "", "Static", ["Surface %d:%d.%d"],@\n',...
        bc(i,1),bc(i,2),bc(i,3));
    fprintf(fileID,'"Geometry", "Coord 0", "1.", ["<0 0 0>", @\n');
    fprintf(fileID,'"", "", ""],@\n');
    fprintf(fileID,' ["", "", "", ""] )\n');
end

% Generate mesh for shell structure
fprintf(fileID,'INTEGER fem_create_mesh_surfa_num_nodes\n');
fprintf(fileID,'INTEGER fem_create_mesh_surfa_num_elems\n');
fprintf(fileID,'STRING fem_create_mesh_s_nodes_created[VIRTUAL]\n');
fprintf(fileID,'STRING fem_create_mesh_s_elems_created[VIRTUAL]\n');

% Collapse fem_dat_sh
fem_dat_sh_coll=collapse(fem_dat_sh);

% Sew surfaces
fprintf(fileID,'STRING sgm_edit_surface_se_surface_ids[VIRTUAL]\n');
fprintf(fileID,'sgm_edit_surface_sew(@\n');
fprintf(fileID,'"Surface 1:%d", sgm_edit_surface_se_surface_ids )\n'...
    ,fem_dat_sh_coll(size(fem_dat_sh_coll,1),2));

% Create shell mesh
for i = 1:size(fem_dat_sh_coll,1)
    if fem_dat_sh_coll(i,4) == 1;
        esize = coursemesh;
    elseif fem_dat_sh_coll(i,4) == 2;
        esize = finemesh;
    end
    fprintf(fileID,'fem_create_mesh_surf_4( "Hybrid", 49664,@\n');
    fprintf(fileID,...
        '"Surface %d:%d", 4, ["%d", "0.1", "0.2", "1.0"],@\n',...
        fem_dat_sh_coll(i,1),fem_dat_sh_coll(i,2),esize);
    fprintf(fileID,'"Quad4", "#", "#", "Coord 0", "Coord 0",@\n');
    fprintf(fileID,'fem_create_mesh_surfa_num_nodes,@\n');
    fprintf(fileID,'fem_create_mesh_surfa_num_elems,@\n');
    fprintf(fileID,'fem_create_mesh_s_nodes_created,@\n');
    fprintf(fileID,'fem_create_mesh_s_elems_created )\n');
    fprintf(fileID,'fem_associate_elems_to_ep( "%s",@\n',...
        char(spname(fem_dat_sh_coll(i,3))));
    fprintf(fileID,'fem_create_mesh_s_elems_created,@\n');
    fprintf(fileID,'fem_create_mesh_surfa_num_elems )\n');
end

%Equivalence nodes
fprintf(fileID,'REAL fem_equiv_all_x_equivtol_ab\n');
fprintf(fileID,'INTEGER fem_equiv_all_x_segment\n');
fprintf(fileID,'fem_equiv_all_group4( [" "], 0, "", @\n');
fprintf(fileID,'1, 1, 10., FALSE, fem_equiv_all_x_equivtol_ab, @\n');
fprintf(fileID,'fem_equiv_all_x_segment )\n');
end